import flet as ft
from flet_ads.base_ad import BaseAd


@ft.control("InterstitialAd")
class InterstitialAd(BaseAd):
    """
    Displays a full-screen interstitial ad.

    Raises:
        FletUnsupportedPlatformException: When using this control on a
            web and/or non-mobile platform.
    """

    async def show(self):
        """
        Present the loaded interstitial ad as a full-screen overlay.

        The ad must be loaded before this method is called.
        """

        await self._invoke_method("show")



from dataclasses import dataclass
from enum import Enum
from typing import Optional

import flet as ft

__all__ = [
    "AdRequest",
    "NativeAdTemplateStyle",
    "NativeAdTemplateTextStyle",
    "NativeAdTemplateType",
    "NativeTemplateFontStyle",
    "PaidAdEvent",
    "PrecisionType",
]


class PrecisionType(Enum):
    """Describes how accurately a paid ad value is reported."""

    UNKNOWN = "unknown"
    """An ad value with unknown precision."""

    ESTIMATED = "estimated"
    """An ad value estimated from aggregated data."""

    PUBLISHER_PROVIDED = "publisherProvided"
    """A publisher-provided ad value, such as manual CPMs in a mediation group."""

    PRECISE = "precise"
    """The precise value paid for this ad."""


@dataclass
class PaidAdEvent(ft.Event[ft.EventControlType]):
    """
    Event data for paid ad events.
    """

    value: float
    """
    The monetary value of the ad.
    """

    precision: PrecisionType
    """
    The precision of the ad value.
    """

    currency_code: str
    """
    The currency code of the ad value.
    """


@dataclass
class AdRequest:
    """
    Targeting info per the AdMob API.

    This class's properties mirror the native AdRequest API. See for example:
    [AdRequest.Builder for Android](https://developers.google.com/android/reference/com/google/android/gms/ads/AdRequest.Builder).
    """

    keywords: Optional[list[str]] = None
    """
    Words or phrases describing the current user activity.
    """

    content_url: Optional[str] = None
    """
    URL string for a webpage whose content matches the app’s primary content.

    This webpage content is used for targeting and brand safety purposes.
    """

    neighboring_content_urls: Optional[list[str]] = None
    """
    URLs representing web content near an ad.
    """

    non_personalized_ads: Optional[bool] = None
    """
    Non-personalized ads are ads that are not based on a user’s past behavior.

    For more information: https://support.google.com/admob/answer/7676680?hl=en
    """

    http_timeout: Optional[int] = None
    """
    A custom timeout (in milliseconds) for HTTPS calls during an ad request.

    Note:
        This is only supported in Android. (ignored on iOS)
    """

    extras: Optional[dict[str, str]] = None
    """
    Extras to pass to the AdMob adapter.
    """


class NativeAdTemplateType(Enum):
    SMALL = "small"
    MEDIUM = "medium"


class NativeTemplateFontStyle(Enum):
    NORMAL = "normal"
    BOLD = "bold"
    ITALIC = "italic"
    MONOSPACE = "monospace"


@dataclass
class NativeAdTemplateTextStyle:
    size: Optional[ft.Number] = None
    text_color: Optional[ft.ColorValue] = None
    bgcolor: Optional[ft.ColorValue] = None
    style: Optional[NativeTemplateFontStyle] = None


@dataclass
class NativeAdTemplateStyle:
    template_type: NativeAdTemplateType = NativeAdTemplateType.MEDIUM
    main_bgcolor: Optional[ft.ColorValue] = None
    corner_radius: Optional[ft.Number] = None
    call_to_action_text_style: Optional[NativeAdTemplateTextStyle] = None
    primary_text_style: Optional[NativeAdTemplateTextStyle] = None
    secondary_text_style: Optional[NativeAdTemplateTextStyle] = None
    tertiary_text_style: Optional[NativeAdTemplateTextStyle] = None



from typing import Optional

import flet as ft
from flet_ads.base_ad import BaseAd
from flet_ads.types import PaidAdEvent


@ft.control("BannerAd")
class BannerAd(BaseAd):
    """
    Displays a banner ad.

    Raises:
        FletUnsupportedPlatformException: When this control is used on a web
            and/or non-mobile platform.
    """

    on_will_dismiss: Optional[ft.ControlEventHandler["BannerAd"]] = None
    """
    Called before dismissing a full screen view.

    Note:
        Only available on iOS.
    """

    on_paid: Optional[ft.ControlEventHandler[PaidAdEvent["BannerAd"]]] = None
    """
    Called when this ad is estimated to have earned money.

    Available for allowlisted accounts only.
    """



from dataclasses import field
from typing import Optional

import flet as ft
from flet_ads.types import AdRequest


@ft.control
class BaseAd(ft.Control):
    """
    Base class for all ad controls in Flet Ads package.

    Raises:
        FletUnsupportedPlatformException: When using this control on a web
            and/or non-mobile platform.
    """

    unit_id: str
    """
    Ad unit ID for this ad.
    """

    request: AdRequest = field(default_factory=lambda: AdRequest())
    """
    Targeting information used to fetch an Ad.
    """

    on_load: Optional[ft.ControlEventHandler["BaseAd"]] = None
    """
    Called when this ad is loaded successfully.
    """

    on_error: Optional[ft.ControlEventHandler["BaseAd"]] = None
    """
    Called when an ad request failed.

    Event handler argument [`data`][flet.Event.data] property
    contains information about the error.
    """

    on_open: Optional[ft.ControlEventHandler["BaseAd"]] = None
    """
    Called when this ad opens up.

    A full screen view/overlay is presented in response to the user clicking
    on an ad. You may want to pause animations and time sensitive
    interactions.
    """

    on_close: Optional[ft.ControlEventHandler["BaseAd"]] = None
    """
    Called when the full screen view has been closed. You should restart
    anything paused while handling [`on_open`][flet_ads.BaseAd.on_open].
    """

    on_impression: Optional[ft.ControlEventHandler["BaseAd"]] = None
    """
    Called when an impression occurs on this ad.
    """

    on_click: Optional[ft.ControlEventHandler["BaseAd"]] = None
    """
    Called when this ad is clicked.
    """

    def before_update(self):
        if self.page.web or not self.page.platform.is_mobile():
            raise ft.FletUnsupportedPlatformException(
                f"{self.__class__.__name__} is only supported on "
                f"Mobile (Android and iOS)"
            )



from flet_ads.banner_ad import BannerAd
from flet_ads.base_ad import BaseAd
from flet_ads.interstitial_ad import InterstitialAd
from flet_ads.types import (
    AdRequest,
    PaidAdEvent,
    PrecisionType,
)

__all__ = [
    "AdRequest",
    "BannerAd",
    "BaseAd",
    "InterstitialAd",
    "PaidAdEvent",
    "PrecisionType",
]



import flet as ft
from flet_ads.banner_ad import BannerAd
from flet_ads.types import NativeAdTemplateStyle

__all__ = ["NativeAd"]


@ft.control("NativeAd")
class NativeAd(BannerAd):
    """
    Renders a native ad.
    """

    factory_id: str = None
    """
    An identifier for the factory that creates the Platform view.

    Raises:
        ValueError: When neither `factory_id` nor [`template_style`][(c).] is set.
    """

    template_style: NativeAdTemplateStyle = None
    """
    A style for the native ad template.

    Raises:
        ValueError: When neither [`factory_id`][(c).] nor `template_style` is set.
    """

    def before_update(self):
        super().before_update()
        if self.factory_id is None and self.template_style is None:
            raise ValueError("factory_id or template_style must be set")
